@@ -31,7 +31,7 @@ extern EventType ET_TASK;
 class TasksProcessor : public Processor
 {
 public:
-  int start(int task_threads, size_t stacksize = DEFAULT_STACKSIZE);
+  int start(int task_threads, size_t stacksize = DEFAULT_STACKSIZE) override;
 };
 
 extern TasksProcessor tasksProcessor;
