@@ -23,8 +23,8 @@
 
 #pragma once
 
-#include "ts/ink_platform.h"
-#include "ts/ink_resource.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_resource.h"
 
 // TODO: I think we're overly aggressive here on making MIOBuffer 64-bit
 // but not sure it's worthwhile changing anything to 32-bit honestly.
@@ -40,29 +40,32 @@ buffer_size_to_index(int64_t size, int64_t max = max_iobuffer_size)
 {
   int64_t r = max;
 
-  while (r && BUFFER_SIZE_FOR_INDEX(r - 1) >= size)
+  while (r && BUFFER_SIZE_FOR_INDEX(r - 1) >= size) {
     r--;
+  }
   return r;
 }
 
 TS_INLINE int64_t
 iobuffer_size_to_index(int64_t size, int64_t max)
 {
-  if (size > BUFFER_SIZE_FOR_INDEX(max))
+  if (size > BUFFER_SIZE_FOR_INDEX(max)) {
     return BUFFER_SIZE_INDEX_FOR_XMALLOC_SIZE(size);
+  }
   return buffer_size_to_index(size, max);
 }
 
 TS_INLINE int64_t
 index_to_buffer_size(int64_t idx)
 {
-  if (BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(idx))
+  if (BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(idx)) {
     return BUFFER_SIZE_FOR_INDEX(idx);
-  else if (BUFFER_SIZE_INDEX_IS_XMALLOCED(idx))
+  } else if (BUFFER_SIZE_INDEX_IS_XMALLOCED(idx)) {
     return BUFFER_SIZE_FOR_XMALLOC(idx);
-  // coverity[dead_error_condition]
-  else if (BUFFER_SIZE_INDEX_IS_CONSTANT(idx))
+    // coverity[dead_error_condition]
+  } else if (BUFFER_SIZE_INDEX_IS_CONSTANT(idx)) {
     return BUFFER_SIZE_FOR_CONSTANT(idx);
+  }
   // coverity[dead_error_line]
   return 0;
 }
@@ -147,27 +150,33 @@ iobufferblock_skip(IOBufferBlock *b, int64_t *poffset, int64_t *plen, int64_t wr
 TS_INLINE void
 iobuffer_mem_inc(const char *_loc, int64_t _size_index)
 {
-  if (!res_track_memory)
+  if (!res_track_memory) {
     return;
+  }
 
-  if (!BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(_size_index))
+  if (!BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(_size_index)) {
     return;
+  }
 
-  if (!_loc)
+  if (!_loc) {
     _loc = "memory/IOBuffer/UNKNOWN-LOCATION";
+  }
   ResourceTracker::increment(_loc, index_to_buffer_size(_size_index));
 }
 
 TS_INLINE void
 iobuffer_mem_dec(const char *_loc, int64_t _size_index)
 {
-  if (!res_track_memory)
+  if (!res_track_memory) {
     return;
+  }
 
-  if (!BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(_size_index))
+  if (!BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(_size_index)) {
     return;
-  if (!_loc)
+  }
+  if (!_loc) {
     _loc = "memory/IOBuffer/UNKNOWN-LOCATION";
+  }
   ResourceTracker::increment(_loc, -index_to_buffer_size(_size_index));
 }
 #endif
@@ -207,20 +216,6 @@ new_IOBufferData_internal(
   return d;
 }
 
-TS_INLINE IOBufferData *
-new_constant_IOBufferData_internal(
-#ifdef TRACK_BUFFER_USER
-  const char *loc,
-#endif
-  void *b, int64_t size)
-{
-  return new_IOBufferData_internal(
-#ifdef TRACK_BUFFER_USER
-    loc,
-#endif
-    b, size, BUFFER_SIZE_INDEX_FOR_CONSTANT_SIZE(size));
-}
-
 TS_INLINE IOBufferData *
 new_xmalloc_IOBufferData_internal(
 #ifdef TRACK_BUFFER_USER
@@ -271,8 +266,9 @@ new_IOBufferData_internal(
 TS_INLINE void
 IOBufferData::alloc(int64_t size_index, AllocType type)
 {
-  if (_data)
+  if (_data) {
     dealloc();
+  }
   _size_index = size_index;
   _mem_type   = type;
 #ifdef TRACK_BUFFER_USER
@@ -280,18 +276,20 @@ IOBufferData::alloc(int64_t size_index, AllocType type)
 #endif
   switch (type) {
   case MEMALIGNED:
-    if (BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(size_index))
+    if (BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(size_index)) {
       _data = (char *)ioBufAllocator[size_index].alloc_void();
-    // coverity[dead_error_condition]
-    else if (BUFFER_SIZE_INDEX_IS_XMALLOCED(size_index))
+      // coverity[dead_error_condition]
+    } else if (BUFFER_SIZE_INDEX_IS_XMALLOCED(size_index)) {
       _data = (char *)ats_memalign(ats_pagesize(), index_to_buffer_size(size_index));
+    }
     break;
   default:
   case DEFAULT_ALLOC:
-    if (BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(size_index))
+    if (BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(size_index)) {
       _data = (char *)ioBufAllocator[size_index].alloc_void();
-    else if (BUFFER_SIZE_INDEX_IS_XMALLOCED(size_index))
+    } else if (BUFFER_SIZE_INDEX_IS_XMALLOCED(size_index)) {
       _data = (char *)ats_malloc(BUFFER_SIZE_FOR_XMALLOC(size_index));
+    }
     break;
   }
 }
@@ -306,20 +304,22 @@ IOBufferData::dealloc()
 #endif
   switch (_mem_type) {
   case MEMALIGNED:
-    if (BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(_size_index))
+    if (BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(_size_index)) {
       ioBufAllocator[_size_index].free_void(_data);
-    else if (BUFFER_SIZE_INDEX_IS_XMALLOCED(_size_index))
+    } else if (BUFFER_SIZE_INDEX_IS_XMALLOCED(_size_index)) {
       ::free((void *)_data);
+    }
     break;
   default:
   case DEFAULT_ALLOC:
-    if (BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(_size_index))
+    if (BUFFER_SIZE_INDEX_IS_FAST_ALLOCATED(_size_index)) {
       ioBufAllocator[_size_index].free_void(_data);
-    else if (BUFFER_SIZE_INDEX_IS_XMALLOCED(_size_index))
+    } else if (BUFFER_SIZE_INDEX_IS_XMALLOCED(_size_index)) {
       ats_free(_data);
+    }
     break;
   }
-  _data       = 0;
+  _data       = nullptr;
   _size_index = BUFFER_SIZE_NOT_ALLOCATED;
   _mem_type   = NO_ALLOC;
 }
@@ -342,7 +342,7 @@ new_IOBufferBlock_internal(
 #ifdef TRACK_BUFFER_USER
   const char *location
 #endif
-  )
+)
 {
   IOBufferBlock *b = THREAD_ALLOC(ioBlockAllocator, this_thread());
 #ifdef TRACK_BUFFER_USER
@@ -368,12 +368,12 @@ new_IOBufferBlock_internal(
 
 TS_INLINE
 IOBufferBlock::IOBufferBlock()
-  : _start(0),
-    _end(0),
-    _buf_end(0)
+  : _start(nullptr),
+    _end(nullptr),
+    _buf_end(nullptr)
 #ifdef TRACK_BUFFER_USER
     ,
-    _location(0)
+    _location(nullptr)
 #endif
 {
   return;
@@ -511,18 +511,6 @@ IOBufferBlock::realloc(void *b, int64_t buf_size)
   realloc_set_internal(b, buf_size, BUFFER_SIZE_NOT_ALLOCATED);
 }
 
-TS_INLINE void
-IOBufferBlock::realloc_xmalloc(void *b, int64_t buf_size)
-{
-  realloc_set_internal(b, buf_size, -buf_size);
-}
-
-TS_INLINE void
-IOBufferBlock::realloc_xmalloc(int64_t buf_size)
-{
-  realloc_set_internal(ats_malloc(buf_size), buf_size, -buf_size);
-}
-
 TS_INLINE void
 IOBufferBlock::realloc(int64_t i)
 {
@@ -578,7 +566,7 @@ TS_INLINE char *
 IOBufferReader::start()
 {
   if (!block) {
-    return 0;
+    return nullptr;
   }
 
   skip_empty_blocks();
@@ -589,7 +577,7 @@ TS_INLINE char *
 IOBufferReader::end()
 {
   if (!block) {
-    return 0;
+    return nullptr;
   }
 
   skip_empty_blocks();
@@ -687,8 +675,9 @@ TS_INLINE char &IOBufferReader::operator[](int64_t i)
   i += start_offset;
   while (b) {
     int64_t bytes = b->read_avail();
-    if (bytes > i)
+    if (bytes > i) {
       return b->start()[i];
+    }
     i -= bytes;
     b = b->next.get();
   }
@@ -788,6 +777,7 @@ new_MIOBuffer_internal(
   b->_location = location;
 #endif
   b->alloc(size_index);
+  b->water_mark = 0;
   return b;
 }
 
@@ -808,6 +798,7 @@ new_empty_MIOBuffer_internal(
 {
   MIOBuffer *b  = THREAD_ALLOC(ioAllocator, this_thread());
   b->size_index = size_index;
+  b->water_mark = 0;
 #ifdef TRACK_BUFFER_USER
   b->_location = location;
 #endif
@@ -824,9 +815,11 @@ TS_INLINE IOBufferReader *
 MIOBuffer::alloc_accessor(MIOBufferAccessor *anAccessor)
 {
   int i;
-  for (i = 0; i < MAX_MIOBUFFER_READERS; i++)
-    if (!readers[i].allocated())
+  for (i = 0; i < MAX_MIOBUFFER_READERS; i++) {
+    if (!readers[i].allocated()) {
       break;
+    }
+  }
 
   // TODO refactor code to return nullptr at some point
   ink_release_assert(i < MAX_MIOBUFFER_READERS);
@@ -843,9 +836,11 @@ TS_INLINE IOBufferReader *
 MIOBuffer::alloc_reader()
 {
   int i;
-  for (i = 0; i < MAX_MIOBUFFER_READERS; i++)
-    if (!readers[i].allocated())
+  for (i = 0; i < MAX_MIOBUFFER_READERS; i++) {
+    if (!readers[i].allocated()) {
       break;
+    }
+  }
 
   // TODO refactor code to return nullptr at some point
   ink_release_assert(i < MAX_MIOBUFFER_READERS);
@@ -867,9 +862,11 @@ TS_INLINE IOBufferReader *
 MIOBuffer::clone_reader(IOBufferReader *r)
 {
   int i;
-  for (i = 0; i < MAX_MIOBUFFER_READERS; i++)
-    if (!readers[i].allocated())
+  for (i = 0; i < MAX_MIOBUFFER_READERS; i++) {
+    if (!readers[i].allocated()) {
       break;
+    }
+  }
 
   // TODO refactor code to return nullptr at some point
   ink_release_assert(i < MAX_MIOBUFFER_READERS);
@@ -922,12 +919,14 @@ MIOBuffer::append_block_internal(IOBufferBlock *b)
     while (b->read_avail()) {
       _writer = b;
       b       = b->next.get();
-      if (!b)
+      if (!b) {
         break;
+      }
     }
   }
-  while (_writer->next && !_writer->write_avail() && _writer->next->read_avail())
+  while (_writer->next && !_writer->write_avail() && _writer->next->read_avail()) {
     _writer = _writer->next;
+  }
 }
 
 TS_INLINE void
@@ -962,14 +961,17 @@ MIOBuffer::append_block(int64_t asize_index)
 TS_INLINE void
 MIOBuffer::add_block()
 {
-  append_block(size_index);
+  if (this->_writer == nullptr || this->_writer->next == nullptr) {
+    append_block(size_index);
+  }
 }
 
 TS_INLINE void
 MIOBuffer::check_add_block()
 {
-  if (!high_water() && current_low_water())
+  if (!high_water() && current_low_water()) {
     add_block();
+  }
 }
 
 TS_INLINE IOBufferBlock *
@@ -1023,9 +1025,10 @@ MIOBuffer::fill(int64_t len)
   while (f < len) {
     _writer->fill(f);
     len -= f;
-    if (len > 0)
+    if (len > 0) {
       _writer = _writer->next;
-    f         = _writer->write_avail();
+    }
+    f = _writer->write_avail();
   }
   _writer->fill(len);
 }
@@ -1034,9 +1037,9 @@ TS_INLINE int
 MIOBuffer::max_block_count()
 {
   int maxb = 0;
-  for (int i = 0; i < MAX_MIOBUFFER_READERS; i++) {
-    if (readers[i].allocated()) {
-      int c = readers[i].block_count();
+  for (auto &reader : readers) {
+    if (reader.allocated()) {
+      int c = reader.block_count();
       if (c > maxb) {
         maxb = c;
       }
@@ -1050,17 +1053,18 @@ MIOBuffer::max_read_avail()
 {
   int64_t s = 0;
   int found = 0;
-  for (int i = 0; i < MAX_MIOBUFFER_READERS; i++) {
-    if (readers[i].allocated()) {
-      int64_t ss = readers[i].read_avail();
+  for (auto &reader : readers) {
+    if (reader.allocated()) {
+      int64_t ss = reader.read_avail();
       if (ss > s) {
         s = ss;
       }
       found = 1;
     }
   }
-  if (!found && _writer)
+  if (!found && _writer) {
     return _writer->read_avail();
+  }
   return s;
 }
 
@@ -1076,18 +1080,6 @@ MIOBuffer::set(void *b, int64_t len)
   init_readers();
 }
 
-TS_INLINE void
-MIOBuffer::set_xmalloced(void *b, int64_t len)
-{
-#ifdef TRACK_BUFFER_USER
-  _writer = new_IOBufferBlock_internal(_location);
-#else
-  _writer          = new_IOBufferBlock_internal();
-#endif
-  _writer->set_internal(b, len, BUFFER_SIZE_INDEX_FOR_XMALLOC_SIZE(len));
-  init_readers();
-}
-
 TS_INLINE void
 MIOBuffer::append_xmalloced(void *b, int64_t len)
 {
@@ -1122,17 +1114,9 @@ MIOBuffer::alloc(int64_t i)
 #endif
   _writer->alloc(i);
   size_index = i;
-  water_mark = 0;
   init_readers();
 }
 
-TS_INLINE void
-MIOBuffer::alloc_xmalloc(int64_t buf_size)
-{
-  char *b = (char *)ats_malloc(buf_size);
-  set_xmalloced(b, buf_size);
-}
-
 TS_INLINE void
 MIOBuffer::dealloc_reader(IOBufferReader *e)
 {
@@ -1159,9 +1143,11 @@ IOBufferReader::dealloc()
 TS_INLINE void
 MIOBuffer::dealloc_all_readers()
 {
-  for (int i = 0; i < MAX_MIOBUFFER_READERS; i++)
-    if (readers[i].allocated())
-      dealloc_reader(&readers[i]);
+  for (auto &reader : readers) {
+    if (reader.allocated()) {
+      dealloc_reader(&reader);
+    }
+  }
 }
 
 TS_INLINE void
@@ -1174,17 +1160,19 @@ TS_INLINE void
 MIOBufferAccessor::reader_for(MIOBuffer *abuf)
 {
   mbuf = abuf;
-  if (abuf)
+  if (abuf) {
     entry = mbuf->alloc_accessor(this);
-  else
+  } else {
     entry = nullptr;
+  }
 }
 
 TS_INLINE void
 MIOBufferAccessor::reader_for(IOBufferReader *areader)
 {
-  if (entry == areader)
+  if (entry == areader) {
     return;
+  }
   mbuf  = areader->mbuf;
   entry = areader;
   ink_assert(mbuf);
@@ -1198,6 +1186,4 @@ MIOBufferAccessor::writer_for(MIOBuffer *abuf)
 }
 
 TS_INLINE
-MIOBufferAccessor::~MIOBufferAccessor()
-{
-}
+MIOBufferAccessor::~MIOBufferAccessor() {}
